//##################################################################
// Assert 32 library
//##################################################################

#DEFINE WINVER NTDDI_WINXP
#DEFINE WIN32_LEAN_AND_MEAN

#include "WINDOWS.H"
#include "Dbghelp.h"

// Symbols require a debug build:
// GoAsm /c /x86 "project.Asm"
// GoLink "project.obj"   /Debug dbg

// operators
// |,&,^,*,/,+,-,!,(,)
// comparisons
// !=,!<,!>,<>,>=,<=,=,<,>

// use signed multiply and divide
// also signed comparisons
#define SIGNEDOP

// The REGISTERS structure must be in the same order as PUSHAD
// in order to use the stack in the macros !
REGISTERS STRUCT
	Edi DD
	Esi DD
	Ebp DD
	Esp DD
	Ebx DD
	Edx DD
	Ecx DD
	Eax DD
ENDS

#define STACKSIZE 60
#define CALC_FAIL -2
#define CALC_SUCCESS -1
#define CALC_DIVBYZERO -3
#define CALC_OVERFLOW -4
#define CALC_SYMNOTFOUND -5

// enum DATATYPE_TAG
DATATYPE_DEFAULT = -1
DATATYPE_BYTE = 0
DATATYPE_WORD = 1
DATATYPE_DWORD = 2
DATATYPE_QWORD = 3

// enum CALC_SYMBOLS_TAG
CALC_SYMBOL_OR=0 // |
CALC_SYMBOL_AND=1 // &
CALC_SYMBOL_XOR=2 // ^
CALC_SYMBOL_MULTIPLY=3 // *
CALC_SYMBOL_DIVIDE=4 // /
CALC_SYMBOL_ADD=5 // +
CALC_SYMBOL_SUBTRACT=6 // -
CALC_SYMBOL_NOT=7 // !
CALC_SYMBOL_OPEN=8  // (
CALC_SYMBOL_CLOSE=9 // )
CALC_SYMBOL_NUMBER=10
CALC_SYMBOL_INVALID=11

STACKENTRY STRUCT
	var INT32
	sym ENUM // CALC_SYMBOLS
ENDS

// ASSERT functions will start with RDBG_ASRT_ so they will not appear in a symbol dump
// ASSERT will have its own library though it will share the Debug32.a include file

// The stack buffer must be the first entry in this structure !
// The other fields are moveable
BUFFERPOINTERS STRUCT
	StackBuffer STACKENTRY STACKSIZE DUP <>
	StackIndex DD
	reg REGISTERS
	OldFS14 DD
	SignedOperations DD
ends

RDBG_ASRT_AllocBuffers equ _RDBG_ASRT_AllocBuffers@0
RDBG_ASRT_DeAllocBuffers equ _RDBG_ASRT_DeAllocBuffers@0
RDBG_ASRT_Assert equ _RDBG_ASRT_Assert@4

VirtualAlloc equ _VirtualAlloc@16
VirtualFree equ _VirtualFree@12

CoTaskMemAlloc equ _CoTaskMemAlloc@4
lstrcpy equ _lstrcpyA@8
SymInitialize equ _SymInitialize@12
SymFromName equ _SymFromName@12
SymCleanup equ _SymCleanup@4
CoTaskMemFree equ _CoTaskMemFree@4

StrToIntExA equ _StrToIntExA@12

GetModuleHandleA equ _GetModuleHandleA@4
LoadIconA equ _LoadIconA@8
LoadCursorA equ _LoadCursorA@8
RegisterClassExA equ _RegisterClassExA@4
CreateWindowExA equ _CreateWindowExA@48
GetStockObject equ _GetStockObject@4
SendDlgItemMessageA equ _SendDlgItemMessageA@20
CreateFontIndirectA equ _CreateFontIndirectA@4
GetModuleFileNameA equ _GetModuleFileNameA@12
PathFindFileNameA equ _PathFindFileNameA@4
SetDlgItemInt equ _SetDlgItemInt@16
ShowWindow equ _ShowWindow@8
UpdateWindow equ _UpdateWindow@4
GetMessageA equ _GetMessageA@16
TranslateMessage equ _TranslateMessage@4
DispatchMessageA equ _DispatchMessageA@4
SetWindowLongA equ _SetWindowLongA@12
PostMessageA equ _PostMessageA@16
DestroyWindow equ _DestroyWindow@4
DeleteObject equ _DeleteObject@4
GetWindowLongA equ _GetWindowLongA@8
PostQuitMessage equ _PostQuitMessage@4
DefWindowProcA equ _DefWindowProcA@16

// internal ones shouldn't have to be defined for MASM AFAIK
RDBG_ASRT_lszCmpi equ _RDBG_ASRT_lszCmpi@8
RDBG_ASRT_a2dw equ _RDBG_ASRT_a2dw@4
RDBG_ASRT_push equ _RDBG_ASRT_push@4
RDBG_ASRT_pop equ _RDBG_ASRT_pop@4
RDBG_ASRT_peek equ _RDBG_ASRT_peek@4
RDBG_ASRT_initstack equ _RDBG_ASRT_initstack@0
RDBG_ASRT_binary_op equ _RDBG_ASRT_binary_op@4
RDBG_ASRT_unary_op equ _RDBG_ASRT_unary_op@4
RDBG_ASRT_paren equ _RDBG_ASRT_paren@4
RDBG_ASRT_number equ _RDBG_ASRT_number@4
RDBG_ASRT_calculate equ _RDBG_ASRT_calculate@8
RDBG_ASRT_ScanForSymbol equ _RDBG_ASRT_ScanForSymbol@8
RDBG_ASRT_FindSymbolByName equ _RDBG_ASRT_FindSymbolByName@16
RDBG_ASRT_ConvertHex equ _RDBG_ASRT_ConvertHex@4

#include "DbgBrkDialog.Asm"

CONST SECTION
align 16
RDBG_ASRT_lcase:
	db   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
	db  16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
	db  32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47
	db  48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63
	db  64, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
	db 112,113,114,115,116,117,118,119,120,121,122, 91, 92, 93, 94, 95
	db  96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
	db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
	db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
	db 144,145,146,147,148,149,150,151,152,153,154,155,156,156,158,159
	db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,173,175
	db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
	db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
	db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
	db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
	db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255

CODE SECTION

// Allocate the necessary memory
_RDBG_ASRT_AllocBuffers@0 FRAME
	LOCAL bullshot:D
	// We need to allocate enough memory for our stack,
	// registers and stack index. To make it simpler we
	// use a structure.

	// The pointer to the memory is held in fs[0x14]. This is
	// the pvArbitrary slot of the TIB. We save the old value
	// then move our pointer in.

	invoke VirtualAlloc,NULL,SIZEOF BUFFERPOINTERS,MEM_COMMIT,PAGE_READWRITE
	fs mov edx,[0x14]
	mov [eax+BUFFERPOINTERS.OldFS14],edx
	fs mov [0x14],eax
	ret
ENDF

_RDBG_ASRT_DeAllocBuffers@0 FRAME
	LOCAL bullshot:D
	// Get our pointer, restore the old value of pvArbitrary then free our buffer
	fs mov edx,[0x14]
	mov eax,[edx+BUFFERPOINTERS.OldFS14]
	fs mov [0x14],eax
	invoke VirtualFree,edx,NULL,MEM_RELEASE
	ret
ENDF

_RDBG_ASRT_Assert@4 FRAME pExpr
	uses edi,esi,ebx
	LOCAL LeftResult:D
	LOCAL RightResult:D
	LOCAL LeftSide[128]:B
	LOCAL RightSide[128]:B

	// zero the buffers
	mov edi,offset LeftSide
	mov ecx,128
	mov eax,0
	rep stosb
	mov edi,offset RightSide
	mov ecx,128
	mov eax,0
	rep stosb

	// See if the expression has 2 parts
	mov esi,[pExpr]
	xor ebx,ebx
	dec ebx
	:
	inc ebx
	mov ax,[esi+ebx]
	cmp ax,"!="
	je >>.NOTEQUAL
	cmp ax,"<>"
	je >>.NOTEQUAL
	cmp ax,"!<"
	je >>.NOTGREATER ;NOTLESS
	cmp ax,"!>"
	je >>.NOTLESS ;NOTGREATER
	cmp ax,">="
	je >>.NOTGREATER ;NOTLESS
	cmp ax,"<="
	je >>.NOTLESS ;NOTGREATER
	cmp al,"="
	je >>.EQUAL
	cmp al,"<"
	je >>.LESS
	cmp al,">"
	je >>.GREATER
	cmp al,0
	jne <

	.ONEPART
	// Solve the expresssion return TRUE, FALSE or FAIL
	invoke RDBG_ASRT_calculate,[pExpr],offset LeftResult
	cmp eax,CALC_SUCCESS
	jne >>.FAIL
	mov eax,TRUE
	cmp D[LeftResult],0
	jne >
	mov eax,FALSE
	:
	ret

	// Split the expression and solve each part separately
	.NOTEQUAL
		// Copy the left side
		mov ecx,ebx
		mov edi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (2 bytes)
		add esi,2
		xor ecx,ecx
		dec ecx
		mov edi,offset RightSide
		:
			inc ecx
			mov al,[esi+ecx]
			test al,al
			jz >
			mov [edi+ecx],al
			jmp <
		:
		inc ecx
		mov B[edi+ecx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if not equal
		// False otherwise
		mov eax,FALSE
		mov edx,[LeftResult]
		mov ecx,[RightResult]
		cmp edx,ecx
		je >
		mov eax,TRUE
		:
		ret

	.NOTGREATER
		// Copy the left side
		mov ecx,ebx
		mov edi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (2 bytes)
		add esi,2
		xor ecx,ecx
		dec ecx
		mov edi,offset RightSide
		:
			inc ecx
			mov al,[esi+ecx]
			test al,al
			jz >
			mov [edi+ecx],al
			jmp <
		:
		inc ecx
		mov B[edi+ecx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if not greater
		// False otherwise
		mov eax,FALSE
		mov edx,[LeftResult]
		mov ecx,[RightResult]

		fs mov ebx,[0x14]
		cmp D[ebx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.NGSIGNED

		.NGUNSIGNED
		cmp edx,ecx
		jb >
			mov eax,TRUE
		:
		ret

		.NGSIGNED
		cmp edx,ecx
		jl >
			// signed
			mov eax,TRUE
		:
		ret


	.NOTLESS
		// Copy the left side
		mov ecx,ebx
		mov edi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (2 bytes)
		add esi,2
		xor ecx,ecx
		dec ecx
		mov edi,offset RightSide
		:
			inc ecx
			mov al,[esi+ecx]
			test al,al
			jz >
			mov [edi+ecx],al
			jmp <
		:
		inc ecx
		mov B[edi+ecx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if not less
		// False otherwise
		mov eax,FALSE
		mov edx,[LeftResult]
		mov ecx,[RightResult]

		fs mov ebx,[0x14]
		cmp D[ebx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.NLSIGNED

		.NLUNSIGNED
		cmp edx,ecx
		ja >
			mov eax,TRUE
		:
		ret

		.NLSIGNED
		cmp edx,ecx
		jg >
			// signed
			mov eax,TRUE
		:
		ret


	.EQUAL
		// Copy the left side
		mov ecx,ebx
		mov edi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (1 bytes)
		inc esi
		xor ecx,ecx
		dec ecx
		mov edi,offset RightSide
		:
			inc ecx
			mov al,[esi+ecx]
			test al,al
			jz >
			mov [edi+ecx],al
			jmp <
		:
		inc ecx
		mov B[edi+ecx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if equal
		// False otherwise
		mov eax,FALSE
		mov edx,[LeftResult]
		mov ecx,[RightResult]
		cmp edx,ecx
		jne >
			mov eax,TRUE
		:
		ret

	.LESS
		// Copy the left side
		mov ecx,ebx
		mov edi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (1 bytes)
		inc esi
		xor ecx,ecx
		dec ecx
		mov edi,offset RightSide
		:
			inc ecx
			mov al,[esi+ecx]
			test al,al
			jz >
			mov [edi+ecx],al
			jmp <
		:
		inc ecx
		mov B[edi+ecx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if less
		// False otherwise
		mov eax,FALSE
		mov edx,[LeftResult]
		mov ecx,[RightResult]

		fs mov ebx,[0x14]
		cmp D[ebx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.LSIGNED

		.LUNSIGNED
		cmp edx,ecx
		jae >
			mov eax,TRUE
		:
		ret

		.LSIGNED
		cmp edx,ecx
		jge >
			// signed
			mov eax,TRUE
		:
		ret


	.GREATER
		// Copy the left side
		mov ecx,ebx
		mov edi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (1 bytes)
		inc esi
		xor ecx,ecx
		dec ecx
		mov edi,offset RightSide
		:
			inc ecx
			mov al,[esi+ecx]
			test al,al
			jz >
			mov [edi+ecx],al
			jmp <
		:
		inc ecx
		mov B[edi+ecx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp eax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if greater
		// False otherwise
		mov eax,FALSE
		mov edx,[LeftResult]
		mov ecx,[RightResult]

		fs mov ebx,[0x14]
		cmp D[ebx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.GSIGNED

		.GUNSIGNED
		cmp edx,ecx
		jbe >
			mov eax,TRUE
		:
		ret

		.GSIGNED
		cmp edx,ecx
		jle >
			// signed
			mov eax,TRUE
		:
		ret

	.FAIL
	// Return whatever error the calculate function returned
	ret
endf

_RDBG_ASRT_lszCmpi@8 FRAME pString1,pString2
	uses ebx,esi,edi

	lea ebx,RDBG_ASRT_lcase
	mov edi, [pString1]
	mov esi, [pString2]
	:
	mov al, [esi]
	xlatb
	mov ah,al
	mov al, [edi]
	xlatb
	add esi,1
	add edi,1
	sub al, ah
	jnz >
	or ah, ah
	jnz <
	:
	movsx eax, al
	test eax,eax
	ret

ENDF

_RDBG_ASRT_a2dw@4 FRAME lpBuffer
	uses esi

	mov esi, [lpBuffer]
	cmp B[esi],"-"
	jne  >S1
		inc esi
		xor eax,eax
		xor edx,edx
		jmp >N1
		:
		lea eax,[eax*4+eax]
		lea eax,[eax*2+edx]
		N1:
		mov dl,[esi]
		inc esi
		sub dl,"0"
		jns <
		neg eax
		ret

	S1:
		xor eax,eax
		xor edx,edx
		jmp >P1
		:
		lea eax,[eax*4+eax]
		lea eax,[eax*2+edx]
		P1:
		mov dl,[esi]
		inc esi
		sub dl,"0"
		jns <
		ret
ENDF

_RDBG_ASRT_push@4 FRAME p1
	uses ebx

	fs mov ebx,[0x14]
	mov ecx,[ebx+BUFFERPOINTERS.StackIndex]
	cmp ecx,STACKSIZE
	jge >.ERROR

		shl ecx,3
		add ecx,ebx


		mov edx,[p1]
		mov eax,[edx+STACKENTRY.var]
		mov [ecx+STACKENTRY.var],eax
		mov eax,[edx+STACKENTRY.sym]
		mov [ecx+STACKENTRY.sym],eax

		inc D[ebx+BUFFERPOINTERS.StackIndex]

		mov eax,1
		ret
	.ERROR
	mov eax,0
	ret
endf

_RDBG_ASRT_pop@4 FRAME p1
	uses ebx

	fs mov ebx,[0x14]
	mov ecx,[ebx+BUFFERPOINTERS.StackIndex]
	cmp ecx,0
	jle >>.ERROR

		dec D[ebx+BUFFERPOINTERS.StackIndex]
		dec ecx
		shl ecx,3
		add ecx,ebx
		mov edx,[p1]

		mov eax,[ecx+STACKENTRY.var]
		mov [edx+STACKENTRY.var],eax
		mov eax,[ecx+STACKENTRY.sym]
		mov [edx+STACKENTRY.sym],eax

		mov eax,1
		ret
	.ERROR
	mov eax,0
	ret
endf

_RDBG_ASRT_peek@4 FRAME p1
	uses ebx

	fs mov ebx,[0x14]
	mov ecx,[ebx+BUFFERPOINTERS.StackIndex]
	cmp ecx,0
	jle >>.ERROR

		dec ecx
		shl ecx,3
		add ecx,ebx
		mov edx,[p1]

		mov eax,[ecx+STACKENTRY.var]
		mov [edx+STACKENTRY.var],eax
		mov eax,[ecx+STACKENTRY.sym]
		mov [edx+STACKENTRY.sym],eax

		mov eax,1
		ret
	.ERROR
	mov eax,0
	ret
endf

_RDBG_ASRT_initstack@0 FRAME
	LOCAL bullshot:D
	push edi
	mov ecx,STACKSIZE
	shl ecx,3
	fs mov edi,[0x14]

	mov eax,0
	rep stosb
	pop edi

	fs mov ebx,[0x14]
	mov D[ebx+BUFFERPOINTERS.StackIndex],0

	ret
ENDF

_RDBG_ASRT_binary_op@4 FRAME sym
	uses ebx
	LOCAL temp:STACKENTRY


	fs mov ebx,[0x14]
	cmp D[ebx+BUFFERPOINTERS.StackIndex],1

	jge >
		mov eax,CALC_FAIL
		ret
	:
	invoke RDBG_ASRT_peek,offset temp
	cmp D[temp.sym],CALC_SYMBOL_NUMBER
	je >
		mov eax,CALC_FAIL
		ret
	:
	mov eax,[sym]
	mov [temp.sym],eax
	invoke RDBG_ASRT_push,offset temp
	test eax,eax
	jnz >
		mov eax,CALC_FAIL
		ret
	:
	mov eax,CALC_SUCCESS
	ret
endf

_RDBG_ASRT_unary_op@4 FRAME sym
	LOCAL temp:STACKENTRY

	mov eax,[sym]
	mov [temp.sym],eax

	invoke RDBG_ASRT_push,offset temp
	test eax,eax
	jnz >
		mov eax,CALC_FAIL
		ret
	:

	mov eax,CALC_SUCCESS
	ret
endf

_RDBG_ASRT_paren@4 FRAME sym
	uses ebx
	LOCAL temp:STACKENTRY
	LOCAL result:D

	mov ebx,[sym]
	cmp ebx,CALC_SYMBOL_OPEN
	jne >.CALC_SYMBOL_CLOSE
		mov [temp.sym],ebx
		invoke RDBG_ASRT_push,offset temp
		test eax,eax
		jz >>.ERROR
		mov eax,CALC_SUCCESS
		ret

	.CALC_SYMBOL_CLOSE
	cmp ebx,CALC_SYMBOL_CLOSE
	jne >>.ERROR
		invoke RDBG_ASRT_pop,offset temp
		test eax,eax
		jz >>.ERROR
		cmp D[temp.sym],CALC_SYMBOL_NUMBER
		jne >>.ERROR
		mov eax,[temp.var]
		mov [result],eax
		invoke RDBG_ASRT_pop,offset temp
		test eax,eax
		jz >>.ERROR
		cmp D[temp.sym],CALC_SYMBOL_OPEN
		jne >>.ERROR
		invoke RDBG_ASRT_number,[result]
		cmp eax,CALC_SUCCESS
		jne >>.ERROR
		mov eax,CALC_SUCCESS
		ret

	.ERROR
	mov eax,CALC_FAIL
	ret
endf

_RDBG_ASRT_number@4 FRAME num
	uses ebx,esi
	LOCAL temp:STACKENTRY
	LOCAL signed:D
	LOCAL precede:D // CALC_SYMBOLS
	LOCAL result:D

	fs mov ebx,[0x14]
	mov eax,[ebx+BUFFERPOINTERS.SignedOperations]
	mov [signed],eax
	cmp D[ebx+BUFFERPOINTERS.StackIndex],1
	jge >
		mov eax,[num]
		mov [temp.var],eax
		mov D[temp.sym],CALC_SYMBOL_NUMBER
		invoke RDBG_ASRT_push,offset temp
		mov eax,CALC_SUCCESS
		ret
	:
	
	
	invoke RDBG_ASRT_peek,offset temp
	mov ebx,[temp.sym]
	mov [precede],ebx
	
	cmp ebx,CALC_SYMBOL_NOT
	jge >>.CALC_SYMBOL_NOT
		// OR,AND,XOR,*,/,+,-
		invoke RDBG_ASRT_pop,offset temp
		test eax,eax
		jnz >
			mov eax,CALC_FAIL
			ret
		:
		invoke RDBG_ASRT_pop,offset temp
		test eax,eax
		jnz >
			mov eax,CALC_FAIL
			ret
		:
		cmp D[temp.sym],CALC_SYMBOL_NUMBER
		je >
			mov eax,CALC_FAIL
			ret
		:
		// Now the calculation
		cmp ebx,CALC_SYMBOL_OR
		jne >.CALC_SYMBOL_AND
			mov eax,[temp.var]
			mov ecx,[num]
			or eax,ecx
			mov [temp.var],eax
			jmp >>.CALCEND

		.CALC_SYMBOL_AND
		cmp ebx,CALC_SYMBOL_AND
		jne >.CALC_SYMBOL_XOR
			mov eax,[temp.var]
			mov ecx,[num]
			and eax,ecx
			mov [temp.var],eax
			jmp >>.CALCEND
		
		.CALC_SYMBOL_XOR
		cmp ebx,CALC_SYMBOL_XOR
		jne >.CALC_SYMBOL_MULTIPLY
			mov eax,[temp.var]
			mov ecx,[num]
			xor eax,ecx
			mov [temp.var],eax
			jmp >>.CALCEND
		
		.CALC_SYMBOL_MULTIPLY
		cmp ebx,CALC_SYMBOL_MULTIPLY
		jne >.CALC_SYMBOL_DIVIDE
			mov eax,[temp.var]
			mov ecx,[num]
			xor edx,edx
			cmp D[signed],TRUE
			jne >.UNSIGNEDMUL
				imul eax,ecx
				jmp >
			.UNSIGNEDMUL
				mul ecx
			:
			test edx,edx
			jz >
				mov eax,CALC_OVERFLOW
				ret
			:
			mov [temp.var],eax
			jmp >>.CALCEND

		.CALC_SYMBOL_DIVIDE
		cmp ebx,CALC_SYMBOL_DIVIDE
		jne >>.CALC_SYMBOL_ADD
			cmp D[num],0
			jne >
				mov eax,CALC_DIVBYZERO
				ret
			:
			mov eax,[temp.var]
			mov ecx,[num]
			xor edx,edx
			cmp D[signed],TRUE
			jne >.UNSIGNEDDIV
				idiv ecx
				jmp >
			.UNSIGNEDDIV
				div ecx
			:
			mov [temp.var],eax
			jmp >>.CALCEND

		.CALC_SYMBOL_ADD
		cmp ebx,CALC_SYMBOL_ADD
		jne >.CALC_SYMBOL_SUBTRACT
			mov eax,[temp.var]
			mov ecx,[num]
			add eax,ecx
			mov [temp.var],eax
			jmp >>.CALCEND

		.CALC_SYMBOL_SUBTRACT
		cmp ebx,CALC_SYMBOL_SUBTRACT
		jne >>.DEFAULT
			mov eax,[temp.var]
			mov ecx,[num]
			sub eax,ecx
			mov [temp.var],eax
			jmp >>.CALCEND

		.CALCEND
			// Push the result onto the stack
			mov D[temp.sym],CALC_SYMBOL_NUMBER
			invoke RDBG_ASRT_push,offset temp
			test eax,eax
			jnz >
				mov eax,CALC_FAIL
				ret
			:
			mov eax,CALC_SUCCESS
			ret

	.CALC_SYMBOL_NOT
	cmp ebx,CALC_SYMBOL_NOT
	jne >>.CALC_SYMBOL_OPEN
		invoke RDBG_ASRT_pop,offset temp
		mov D[result],TRUE
		cmp D[num],0
		je >
			mov D[result],FALSE
		:
		invoke RDBG_ASRT_number,[result]
		ret

	.CALC_SYMBOL_OPEN
	cmp ebx,CALC_SYMBOL_OPEN
	jne >>.CALC_SYMBOL_CLOSE
		mov eax,[num]
		mov [temp.var],eax
		mov D[temp.sym],CALC_SYMBOL_NUMBER
		invoke RDBG_ASRT_push,offset temp
		test eax,eax
		jnz >
			mov eax,CALC_FAIL
			ret
		:
		mov eax,CALC_SUCCESS
		ret

	.CALC_SYMBOL_CLOSE
	cmp ebx,CALC_SYMBOL_CLOSE
	jne >>.CALC_SYMBOL_NUMBER
		// Not possible
		mov eax,CALC_FAIL
		ret

	.CALC_SYMBOL_NUMBER
	cmp ebx,CALC_SYMBOL_NUMBER
	jne >>.DEFAULT
		// Not possible
		mov eax,CALC_FAIL
		ret

	.DEFAULT
		mov eax,CALC_FAIL
		ret

	ret
endf

_RDBG_ASRT_calculate@8 FRAME exp,result
	uses edi,esi,ebx
	LOCAL temp:STACKENTRY
	LOCAL szNumber[64]:B

	call RDBG_ASRT_initstack
	
	mov edi,[exp]
	test edi,edi
	jnz >
		// bad pointer return 0
		mov eax,0
		ret
	:
	mov eax,[edi]
	test eax,eax
	jnz >
		// empty string
		mov eax,0
		ret
	:
	// set the index to zero
	xor ebx,ebx
	dec ebx
	.WALKEXPRESSION
		inc ebx
		mov esi,[edi+ebx]
		mov ecx,esi
		and esi,0xff
		test esi,esi
		jz >>.ENDWALK

		.OR
		cmp esi,'|'
		jne >>.AND
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_OR
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			// If it fails return the index
			mov eax,ebx
			ret

		.AND
		cmp esi,'&'
		jne >>.XOR
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_AND
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.XOR
		cmp esi,'^'
		jne >>.NOT
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_XOR
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.NOT
		cmp esi,'!'
		jne >>.OPEN
			invoke RDBG_ASRT_unary_op,CALC_SYMBOL_NOT
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.OPEN
		cmp esi,'('
		jne >>.CLOSE
			invoke RDBG_ASRT_paren,CALC_SYMBOL_OPEN
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.CLOSE
		cmp esi,')'
		jne >>.MULTIPLY
			invoke RDBG_ASRT_paren,CALC_SYMBOL_CLOSE
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.MULTIPLY
		cmp esi,'*'
		jne >>.DIVIDE
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_MULTIPLY
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.DIVIDE
		cmp esi,'/'
		jne >>.ADD
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_DIVIDE
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.ADD
		cmp esi,'+'
		jne >>.SUBTRACT
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_ADD
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.SUBTRACT
		cmp esi,'-'
		jne >>.SPACE

			fs mov edx,[0x14]
			cmp D[edx+BUFFERPOINTERS.StackIndex],0
			je >>.NUMBER
			invoke RDBG_ASRT_peek,offset temp
			cmp D[temp.sym],CALC_SYMBOL_NUMBER
			jne >>.NUMBER
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_SUBTRACT
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov eax,ebx
			ret

		.SPACE
		cmp esi," "
		je <<.WALKEXPRESSION

		.TAB
		cmp esi,9
		je <<.WALKEXPRESSION

		.NUMBER
			mov eax,offset szNumber
			cmp esi,"0"
			jne >
				// This may be a HEX number check the next character for X or x
				// ECX contains the next 4 bytes of the expression
				shr ecx,8
				and ecx,0xFF
				or ecx,0x20
				cmp ecx,"x"
				je >>.ISHEXNUMBER
			:
			cmp esi,"-"
			jne >
				mov B[eax],"-"
				inc eax
				inc ebx
				mov esi,[edi+ebx]
				and esi,0xff
			:
			cmp esi,0x30
			jl >>.SYMBOL
			cmp esi,0x39
			jg >>.SYMBOL
			// we have a number copy it to the buffer
				:
				mov cl,[edi+ebx]
				cmp cl,0x39
				jg >
				cmp cl,0x30
				jl >
				mov [eax],cl
				inc ebx
				inc eax
				jmp <
				:
			dec ebx
			mov B[eax],0
			invoke RDBG_ASRT_a2dw,offset szNumber
			invoke RDBG_ASRT_number,eax
			cmp eax,CALC_SUCCESS

			je <<.WALKEXPRESSION
			mov ecx,[result]
			mov D[ecx],eax
			mov eax,ebx
			ret

			.ISHEXNUMBER
			// The number is in HEX format copy it to the buffer
			// call the hex convertor. On return the number will be in EAX
			// the length will be in ECX, ECX will be -1 if an invalid number
			// has been entered (ie > 0xFFFFFFFF or < 0x0)

			// Skip past the 0x
			add ebx,2
			mov ecx,edi
			add ecx,ebx

			invoke RDBG_ASRT_ConvertHex,ecx
			test ecx,ecx
			js >>.INVALID
			add ebx,ecx
			dec ebx

			invoke RDBG_ASRT_number,eax
			cmp eax,CALC_SUCCESS

			je <<.WALKEXPRESSION
			mov ecx,[result]
			mov D[ecx],eax
			mov eax,ebx
			ret

		.SYMBOL
			// the first letter didn't match any of the normal expected ones
			// so we'll see if it's in the symbol table

			// Symbols must start with a letter
			// To get the full symbol name scan ahead until
			// we find an operator. This will be done in a subroutine
			// it will return CALC_SUCCESS if the symbol is found
			// CALC_FAIL if not found
			// The current value of the symbol is in temp.var
			// ecx will contain the count of characters in the symbol so
			// the index can be advanced
			lea eax,[edi+ebx]
			invoke RDBG_ASRT_ScanForSymbol, eax, offset temp
			cmp eax,CALC_FAIL
			jne >
				mov eax,CALC_SYMNOTFOUND
				ret
			:
			add ebx,ecx
			dec ebx
			invoke RDBG_ASRT_number,[temp.var]
			cmp eax,CALC_SUCCESS
			je <<.WALKEXPRESSION

			mov ecx,[result]
			mov D[ecx],eax
			mov eax,ebx
			ret

		.INVALID
			mov eax,ebx
			ret

	.ENDWALK

	fs mov ebx,[0x14]
	cmp D[ebx+BUFFERPOINTERS.StackIndex],1
	je >
		mov eax,ebx
		ret
	:
	invoke RDBG_ASRT_pop,offset temp
	test eax,eax
	jnz >
		mov eax,ebx
		ret
	:
	cmp D[temp.sym],CALC_SYMBOL_NUMBER
	je >
		mov eax,ebx
		ret
	:
	mov eax,[result]
	mov ecx,[temp.var]
	mov [eax],ecx
	mov eax,CALC_SUCCESS
	ret
endf

_RDBG_ASRT_ConvertHex@4 FRAME pStartOfHex
	uses edi,esi,ebx,edx
	LOCAL pszHex[256]:B
	LOCAL intRet:Q

	// Zero the buffer
;	lea edi,pszHex
;	mov al,0
;	mov ecx,255
;	rep stosb

	xor ebx,ebx
	mov esi,[pStartOfHex]
	lea edi,pszHex

	// Copy the 0x
	mov eax,"0x"
	mov [edi],eax
	add edi,2
	xor eax,eax

	:
	mov al,[esi+ebx]
	cmp al,"0"
	jl >.BADCHAR
	cmp al,"9"
	jle >.GOODCHAR
	or al,0x20

	cmp al,"a"
	jl >.BADCHAR
	cmp al,"f"
	jg >.BADCHAR

	.GOODCHAR
	mov [edi+ebx],al
	inc ebx
	jmp <

	.BADCHAR
	mov B[edi+ebx],0
	// The string is packed
	// check the length
	cmp ebx,8
	jg >>.INVALIDNUMBER
	test ebx,ebx
	jz >>.INVALIDNUMBER

	// Push the length
	push ebx

	invoke StrToIntExA,offset pszHex,1,offset intRet

	// Pop the length
	pop ecx
	test eax,eax
	jz >.INVALIDNUMBER

	mov eax,[intRet]
	ret

	.INVALIDNUMBER
	xor eax,eax
	mov ecx,-1
	ret

endf

_RDBG_ASRT_ScanForSymbol@8 FRAME pExpr, p1
	uses ebx,edi,esi
	LOCAL szSymbolName[256]:B
	LOCAL pSymbol:D
	LOCAL fByValue:D
	LOCAL DataType:D

	mov D[DataType],DATATYPE_DEFAULT

	// Optional type indicators (B,W,D) (BYTE,WORD and DWORD respectively)
	// types can only be used on bracketed labels []. Any other use will
	// cause the evaluation to fail. The open bracket must be immediately
	// following the type indicator, no spaces are allowed between them

	// Brackets are not allowed with registers (yet)

	// no brackets [] indicate that the address is to
	// be used instead of the value when dealing with symbols
	// registers will always return the value and cannot be
	// dereferenced (yet)

	mov D[fByValue],FALSE

	// Scan ahead to the next whitespace or operator
	mov edi,[pExpr]
	
	mov al,[edi]
	cmp al,"["
	jne >
		mov D[fByValue],TRUE
		inc edi
		jmp >>.BREAK
	:
	// B W D
	cmp al,"B"
	jne >
		mov al,[edi+1]
		cmp al,"["
		jne >
			mov D[fByValue],TRUE
			mov D[DataType],DATATYPE_BYTE
			add edi,2
			jmp >>.BREAK
	:
	cmp al,"W"
	jne >
		mov al,[edi+1]
		cmp al,"["
		jne >
			mov D[fByValue],TRUE
			mov D[DataType],DATATYPE_WORD
			add edi,2
			jmp >>.BREAK
	:
	cmp al,"D"
	jne >.BREAK
		mov al,[edi+1]
		cmp al,"["
		jne >.BREAK
			mov D[fByValue],TRUE
			mov D[DataType],DATATYPE_DWORD
			add edi,2
	.BREAK
	
	push edi
	xor ecx,ecx

	:
		mov al,[edi]
		inc edi
		inc ecx
		test al,al
		jz >
		cmp al,"+"
		je >
		cmp al,"-"
		je >
		cmp al,"*"
		je >
		cmp al,"/"
		je >
		cmp al,"&"
		je >
		cmp al,"|"
		je >
		cmp al,"^"
		je >
		cmp al,"!"
		je >
		cmp al," "
		je >
		cmp al,9
		je >
		cmp al,"]"
		jne <
	:

	pop esi // ESI points to the source string for movsb
	dec ecx // the last character is the delimiter, don't copy it
	push ecx // Save the length of the symbol for later
	lea edi,szSymbolName // EDI points to the destination string for movsb
	rep movsb // copy the symbol name to a temporary buffer
	mov B[edi],0 // NULL terminate the string

	// Check to see if the symbol is TRUE or FALSE
	// They are supported as FALSE = 0, TRUE = 1
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"TRUE"
	test eax,eax
	jnz >
		pop ecx
		mov edx,[p1]
		mov D[edx+STACKENTRY.var],1
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"FALSE"
	jnz >
		pop ecx
		mov edx,[p1]
		mov D[edx+STACKENTRY.var],0
		mov eax,CALC_SUCCESS
		ret
	:

	// Scan for register names
	// Registers are saved in a context structure before calculate is called

	// Get the offset of the register values
	fs mov edi,[0x14]
	add edi,BUFFERPOINTERS.reg

	// 8 bit registers
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"AL"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Eax]
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"BL"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ebx]
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"CL"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ecx]
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"DL"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Edx]
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"AH"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Eax]
		shr eax,8
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"BH"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ebx]
		shr eax,8
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"CH"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ecx]
		shr eax,8
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"DH"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Edx]
		shr eax,8
		and eax,0xFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:

	// 16 bit registers
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"AX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Eax]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"BX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ebx]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"CX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ecx]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"DX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Edx]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"SI"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Esi]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"DI"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Edi]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"SP"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Esp]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"BP"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ebp]
		and eax,0xFFFF
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:

	// 32 bit registers
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"EAX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Eax]
		cmp D[fByValue],FALSE
		je >.EAXBYVALUE
			mov eax,[eax]
			add ecx,2
		cmp D[DataType],DATATYPE_BYTE
		jne >>.EAXWORD
			and eax,0xFF
			inc ecx
			jmp >.EAXBYVALUE
		.EAXWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.EAXDWORD
			and eax,0xFFFF
			inc ecx
			jmp >.EAXBYVALUE
		.EAXDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.EAXBYVALUE
			inc ecx
		.EAXBYVALUE
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"EBX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ebx]
		
		cmp D[fByValue],FALSE
		je >.EBXBYVALUE
			mov eax,[eax]
			add ecx,2

		cmp D[DataType],DATATYPE_BYTE
		jne >>.EBXWORD
			and eax,0xFF
			inc ecx
			jmp >.EBXBYVALUE
		.EBXWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.EBXDWORD
			and eax,0xFFFF
			inc ecx
			jmp >.EBXBYVALUE
		.EBXDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.EBXBYVALUE
			inc ecx
		.EBXBYVALUE

		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"ECX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ecx]

		cmp D[fByValue],FALSE
		je >.ECXBYVALUE
			mov eax,[eax]
			add ecx,2
		cmp D[DataType],DATATYPE_BYTE
		jne >>.ECXWORD
			and eax,0xFF
			inc ecx
			jmp >.ECXBYVALUE
		.ECXWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.ECXDWORD
			and eax,0xFFFF
			inc ecx
			jmp >.ECXBYVALUE
		.ECXDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.ECXBYVALUE
			inc ecx
		.ECXBYVALUE
		
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"EDX"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Edx]
		
		cmp D[fByValue],FALSE
		je >.EDXBYVALUE
			mov eax,[eax]
			add ecx,2
		cmp D[DataType],DATATYPE_BYTE
		jne >>.EDXWORD
			and eax,0xFF
			inc ecx
			jmp >.EDXBYVALUE
		.EDXWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.EDXDWORD
			and eax,0xFFFF
			inc ecx
			jmp >.EDXBYVALUE
		.EDXDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.EDXBYVALUE
			inc ecx
		.EDXBYVALUE
		
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"ESI"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Esi]

		cmp D[fByValue],FALSE
		je >.ESIBYVALUE
			mov eax,[eax]
			add ecx,2
		cmp D[DataType],DATATYPE_BYTE
		jne >>.ESIWORD
			and eax,0xFF
			inc ecx
			jmp >.ESIBYVALUE
		.ESIWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.ESIDWORD
			and eax,0xFFFF
			inc ecx
			jmp >.ESIBYVALUE
		.ESIDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.ESIBYVALUE
			inc ecx
		.ESIBYVALUE
		
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"EDI"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Edi]

		cmp D[fByValue],FALSE
		je >.EDIBYVALUE
			mov eax,[eax]
			add ecx,2
		cmp D[DataType],DATATYPE_BYTE
		jne >>.EDIWORD
			and eax,0xFF
			inc ecx
			jmp >.EDIBYVALUE
		.EDIWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.EDIDWORD
			and eax,0xFFFF
			inc ecx
			jmp >.EDIBYVALUE
		.EDIDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.EDIBYVALUE
			inc ecx
		.EDIBYVALUE
		
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"ESP"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Esp]

		cmp D[fByValue],FALSE
		je >.ESPBYVALUE
			mov eax,[eax]
			add ecx,2
		cmp D[DataType],DATATYPE_BYTE
		jne >>.ESPWORD
			and eax,0xFF
			inc ecx
			jmp >.ESPBYVALUE
		.ESPWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.ESPDWORD
			and eax,0xFFFF
			inc ecx
			jmp >.ESPBYVALUE
		.ESPDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.ESPBYVALUE
			inc ecx
		.ESPBYVALUE
		
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	invoke RDBG_ASRT_lszCmpi,offset szSymbolName,"EBP"
	jnz >
		pop ecx
		mov edx,[p1]
		mov eax,[edi+REGISTERS.Ebp]

		cmp D[fByValue],FALSE
		je >.EBPBYVALUE
			mov eax,[eax]
			add ecx,2
		cmp D[DataType],DATATYPE_BYTE
		jne >>.EBPWORD
			and eax,0xFF
			jmp >.EBPBYVALUE
		.EBPWORD
		cmp D[DataType],DATATYPE_WORD
		jne >>.EBPDWORD
			and eax,0xFFFF
			jmp >.EBPBYVALUE
		.EBPDWORD
		cmp D[DataType],DATATYPE_DWORD
		jne >>.EBPBYVALUE
			inc ecx
		.EBPBYVALUE
		
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:

	.SKIPSEARCH

	// We need the length of the symbol again so pop it into ECX
	pop ecx

	// ECX is preserved by the RDBG_ASRT_FindSymbolByName procedure
	// It will contain the length of the symbol to advance the
	// index in the calculate procedure
	invoke RDBG_ASRT_FindSymbolByName,-1,offset szSymbolName,offset pSymbol,ecx
	test eax,eax // Returns 0 for success, -1 if the sym isn't found
	js >>.ERROR
	mov eax,[pSymbol]
	cmp D[fByValue],FALSE
	je >
		// The value not the address is required, dereference it and
		// adjust the value of the character count to reflect it
		mov eax,[eax]
		add ecx,2 // skip over the two [] brackets
	:

	cmp D[DataType],DATATYPE_BYTE
	jne >
		mov edx,[p1]
		and eax,0xFF
		inc ecx
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	cmp D[DataType],DATATYPE_WORD
	jne >
		mov edx,[p1]
		and eax,0xFFFF
		inc ecx
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:
	cmp D[DataType],DATATYPE_DWORD
	jne >
		mov edx,[p1]
		inc ecx
		mov [edx+STACKENTRY.var],eax
		mov eax,CALC_SUCCESS
		ret
	:

	// DATATYPE_DEFAULT
	// Put the value into the structure and return success
	// On exit ECX will contain the character count
	mov edx,[p1]
	mov [edx+STACKENTRY.var],eax
	mov eax,CALC_SUCCESS
	ret

	.ERROR
	mov eax,CALC_FAIL
	ret
ENDF

_RDBG_ASRT_FindSymbolByName@16 FRAME hProcess,pszSymbol,ppAddress,lenSymbol
	uses ebx,edi,esi,ecx
	LOCAL pSymInfo:D
	LOCAL SymModInfo:IMAGEHLP_MODULE64

	invoke CoTaskMemAlloc,SIZEOF SYMBOL_INFO+MAX_SYM_NAME-4
	mov [pSymInfo],eax

	mov ebx,[lenSymbol]
	mov D[eax+SYMBOL_INFO.NameLen],ebx
	mov D[eax+SYMBOL_INFO.MaxNameLen],MAX_SYM_NAME
	mov D[eax+SYMBOL_INFO.SizeOfStruct],SIZEOF SYMBOL_INFO

	mov ebx,[pSymInfo]
	add ebx,SYMBOL_INFO.Name
	invoke lstrcpy,ebx,[pszSymbol]

	 // invade process, load all modules
	invoke SymInitialize,[hProcess],0, TRUE

	invoke SymFromName,[hProcess],ebx,[pSymInfo]
	test eax,eax
	jz >>.NOSYM
	mov eax,[pSymInfo]
	mov edx,[eax+SYMBOL_INFO.Address+4]
	mov eax,[eax+SYMBOL_INFO.Address]
	mov ecx,[ppAddress]
	mov [ecx],eax

	// Shut it down
	invoke SymCleanup,[hProcess]
	invoke CoTaskMemFree,[pSymInfo]

	xor eax,eax
	RET

	.NOSYM
	invoke SymCleanup,[hProcess]
	invoke CoTaskMemFree,[pSymInfo]
	mov ecx,[ppAddress]
	mov D[ecx],0
	xor eax,eax
	dec eax
	RET
endf
