//##################################################################
// Assert 64 library
//##################################################################

#DEFINE WINVER NTDDI_WINXP
#DEFINE WIN64_LEAN_AND_MEAN

#DEFINE WIN64

#include "WINDOWS.H"
#include "Dbghelp.h"
#include "Psapi.h"

#include "DbgBreakDialog.asm"

// Symbols require a debug build:
// GoAsm /c /X64 "project.Asm"
// GoLink "project.obj"   /Debug coff

// operators
// |,&,^,*,/,+,-,!,(,)
// comparisons
// !=,!<,!>,<>,>=,<=,=,<,>

// use signed multiply and divide
// also signed comparisons
#define SIGNEDOP

REGISTERS STRUCT
	R15 DQ
	R14 DQ
	R13 DQ
	R12 DQ
	R11 DQ
	R10 DQ
	R9  DQ
	R8	DQ
	Rdi DQ
	Rsi DQ
	Rbp DQ
	Rsp DQ
	Rbx DQ
	Rdx DQ
	Rcx DQ
	Rax DQ
ENDS

#define STACKSIZE 60
#define CALC_FAIL -2
#define CALC_SUCCESS -1
#define CALC_DIVBYZERO -3
#define CALC_OVERFLOW -4
#define CALC_SYMNOTFOUND -5

// enum DATATYPE_TAG
DATATYPE_DEFAULT = -1
DATATYPE_BYTE = 0
DATATYPE_WORD = 1
DATATYPE_DWORD = 2
DATATYPE_QWORD = 3

// enum CALC_SYMBOLS_TAG
CALC_SYMBOL_OR=0 // |
CALC_SYMBOL_AND=1 // &
CALC_SYMBOL_XOR=2 // ^
CALC_SYMBOL_MULTIPLY=3 // *
CALC_SYMBOL_DIVIDE=4 // /
CALC_SYMBOL_ADD=5 // +
CALC_SYMBOL_SUBTRACT=6 // -
CALC_SYMBOL_NOT=7 // !
CALC_SYMBOL_OPEN=8  // (
CALC_SYMBOL_CLOSE=9 // )
CALC_SYMBOL_NUMBER=10
CALC_SYMBOL_INVALID=11

STACKENTRY STRUCT
	var DQ
	sym DQ // CALC_SYMBOLS
ENDS

// ASSERT functions will start with RDBG_ASRT_ so they will not appear in a symbol dump
// ASSERT will have its own library though it will share the Debug64.a include file

// The stack buffer must be the first entry in this structure !
// The other fields are moveable
BUFFERPOINTERS STRUCT
	StackBuffer STACKENTRY STACKSIZE DUP <>
	reg REGISTERS <>
	OldGS28 DQ
	StackIndex DD
	SignedOperations DD
ends

CONST SECTION
align 16
RDBG_ASRT_lcase:
	db   0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
	db  16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
	db  32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47
	db  48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63
	db  64, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
	db 112,113,114,115,116,117,118,119,120,121,122, 91, 92, 93, 94, 95
	db  96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111
	db 112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127
	db 128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143
	db 144,145,146,147,148,149,150,151,152,153,154,155,156,156,158,159
	db 160,161,162,163,164,165,166,167,168,169,170,171,172,173,173,175
	db 176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191
	db 192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207
	db 208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223
	db 224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239
	db 240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255

CODE SECTION
// Allocate the necessary memory
RDBG_ASRT_AllocBuffers FRAME
	LOCAL bullshot:D
	// We need to allocate enough memory for our stack,
	// registers and stack index. To make it simpler we
	// use a structure.

	// The pointer to the memory is held in fs[0x14]. This is
	// the pvArbitrary slot of the TIB. We save the old value
	// then move our pointer in.

	// Save the volatile registers
	push r8
	push r9
	push r10
	push r11
	push rax
	push rcx
	push rdx
	push rbx

	invoke VirtualAlloc,NULL,SIZEOF BUFFERPOINTERS,MEM_COMMIT,PAGE_READWRITE
	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	mov [rax+BUFFERPOINTERS.OldGS28],rbx
	DB 0x65,0x48,0x89,0x04,0x25,0x28,0x00,0x00,0x00 ;gs mov [0x28],rax

	pop rbx
	pop rdx
	pop rcx
	pop rax
	pop r11
	pop r10
	pop r9
	pop r8

	ret
ENDF

RDBG_ASRT_DeAllocBuffers FRAME
	LOCAL bullshot:D
	push rbx
	// Get our pointer, restore the old value of pvArbitrary then free our buffer
	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	mov rax,[rbx+BUFFERPOINTERS.OldGS28]
	DB 0x65,0x48,0x89,0x04,0x25,0x28,0x00,0x00,0x00 ;gs mov [0x28],rax
	invoke VirtualFree,rbx,NULL,MEM_RELEASE
	pop rbx
	ret
ENDF

RDBG_ASRT_Assert FRAME pExpr
	uses rdi,rsi,rbx
	LOCAL LeftResult:Q
	LOCAL RightResult:Q
	LOCAL LeftSide[128]:B
	LOCAL RightSide[128]:B

	// zero the buffers
	mov rdi,offset LeftSide
	mov rcx,128
	mov rax,0
	rep stosb
	mov rdi,offset RightSide
	mov rcx,128
	mov rax,0
	rep stosb

	// See if the expression has 2 parts
	mov rsi,[pExpr]
	xor rbx,rbx
	dec rbx
	:
	inc rbx
	mov ax,[rsi+rbx]
	cmp ax,"!="
	je >>.NOTEQUAL
	cmp ax,"<>"
	je >>.NOTEQUAL
	cmp ax,"!<"
	je >>.NOTGREATER ;NOTLESS
	cmp ax,"!>"
	je >>.NOTLESS ;NOTGREATER
	cmp ax,">="
	je >>.NOTGREATER ;NOTLESS
	cmp ax,"<="
	je >>.NOTLESS ;NOTGREATER
	cmp al,"="
	je >>.EQUAL
	cmp al,"<"
	je >>.LESS
	cmp al,">"
	je >>.GREATER
	cmp al,0
	jne <

	.ONEPART
	// Solve the expresssion return TRUE, FALSE or FAIL
	invoke RDBG_ASRT_calculate,[pExpr],offset LeftResult
	cmp rax,CALC_SUCCESS
	jne >>.FAIL
	mov rax,TRUE
	cmp Q[LeftResult],0
	jne >
	mov rax,FALSE
	:
	ret

	// Split the expression and solve each part separately
	.NOTEQUAL
		// Copy the left side
		mov rcx,rbx
		mov rdi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (2 bytes)
		add rsi,2
		xor rcx,rcx
		dec rcx
		mov rdi,offset RightSide
		:
			inc rcx
			mov al,[rsi+rcx]
			test al,al
			jz >
			mov [rdi+rcx],al
			jmp <
		:
		inc rcx
		mov B[rdi+rcx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if not equal
		// False otherwise
		mov rax,FALSE
		mov rdx,[LeftResult]
		mov rcx,[RightResult]
		cmp rdx,rcx
		je >
		mov rax,TRUE
		:
		ret

	.NOTGREATER
		// Copy the left side
		mov rcx,rbx
		mov rdi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (2 bytes)
		add rsi,2
		xor rcx,rcx
		dec rcx
		mov rdi,offset RightSide
		:
			inc rcx
			mov al,[rsi+rcx]
			test al,al
			jz >
			mov [rdi+rcx],al
			jmp <
		:
		inc rcx
		mov B[rdi+rcx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if not greater
		// False otherwise
		mov rax,FALSE
		mov rdx,[LeftResult]
		mov rcx,[RightResult]

		DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]

		cmp D[rbx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.NGSIGNED

		.NGUNSIGNED
		cmp rdx,rcx
		jb >
			mov rax,TRUE
		:
		ret

		.NGSIGNED
		cmp rdx,rcx
		jl >
			// signed
			mov rax,TRUE
		:
		ret


	.NOTLESS
		// Copy the left side
		mov rcx,rbx
		mov rdi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (2 bytes)
		add rsi,2
		xor rcx,rcx
		dec rcx
		mov rdi,offset RightSide
		:
			inc rcx
			mov al,[rsi+rcx]
			test al,al
			jz >
			mov [rdi+rcx],al
			jmp <
		:
		inc rcx
		mov B[rdi+rcx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if not less
		// False otherwise
		mov rax,FALSE
		mov rdx,[LeftResult]
		mov rcx,[RightResult]

		DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
		cmp D[rbx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.NLSIGNED

		.NLUNSIGNED
		cmp rdx,rcx
		ja >
			mov rax,TRUE
		:
		ret

		.NLSIGNED
		cmp rdx,rcx
		jg >
			// signed
			mov rax,TRUE
		:
		ret


	.EQUAL
		// Copy the left side
		mov rcx,rbx
		mov rdi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (1 bytes)
		inc rsi
		xor rcx,rcx
		dec rcx
		mov rdi,offset RightSide
		:
			inc rcx
			mov al,[rsi+rcx]
			test al,al
			jz >
			mov [rdi+rcx],al
			jmp <
		:
		inc rcx
		mov B[rdi+rcx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if equal
		// False otherwise
		mov rax,FALSE
		mov rdx,[LeftResult]
		mov rcx,[RightResult]
		cmp rdx,rcx
		jne >
			mov rax,TRUE
		:
		ret

	.LESS
		// Copy the left side
		mov rcx,rbx
		mov rdi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (1 bytes)
		inc rsi
		xor rcx,rcx
		dec rcx
		mov rdi,offset RightSide
		:
			inc rcx
			mov al,[rsi+rcx]
			test al,al
			jz >
			mov [rdi+rcx],al
			jmp <
		:
		inc rcx
		mov B[rdi+rcx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if less
		// False otherwise
		mov rax,FALSE
		mov rdx,[LeftResult]
		mov rcx,[RightResult]

		DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
		cmp D[rbx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.LSIGNED

		.LUNSIGNED
		cmp rdx,rcx
		jae >
			mov rax,TRUE
		:
		ret

		.LSIGNED
		cmp rdx,rcx
		jge >
			// signed
			mov rax,TRUE
		:
		ret


	.GREATER
		// Copy the left side
		mov rcx,rbx
		mov rdi,offset LeftSide
		rep movsb
		// Copy the right side - skip past the equate operator (1 bytes)
		inc rsi
		xor rcx,rcx
		dec rcx
		mov rdi,offset RightSide
		:
			inc rcx
			mov al,[rsi+rcx]
			test al,al
			jz >
			mov [rdi+rcx],al
			jmp <
		:
		inc rcx
		mov B[rdi+rcx],0
		invoke RDBG_ASRT_calculate,offset LeftSide,offset LeftResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		invoke RDBG_ASRT_calculate,offset RightSide,offset RightResult
		cmp rax,CALC_SUCCESS
		jne >>.FAIL
		// Return TRUE if greater
		// False otherwise
		mov rax,FALSE
		mov rdx,[LeftResult]
		mov rcx,[RightResult]

		DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
		cmp D[ebx+BUFFERPOINTERS.SignedOperations],TRUE
		je >>.GSIGNED

		.GUNSIGNED
		cmp rdx,rcx
		jbe >
			mov rax,TRUE
		:
		ret

		.GSIGNED
		cmp rdx,rcx
		jle >
			// signed
			mov rax,TRUE
		:
		ret

	.FAIL

	// Return whatever error the calculate function returned
	// in RBX, set RAX to -1 to signal an expression error
	mov rcx,rax
	mov rax,-1
	ret
endf

RDBG_ASRT_lszCmpi FRAME pString1,pString2
	uses ebx,esi,edi

	lea rbx,RDBG_ASRT_lcase
	mov rdi, [pString1]
	mov rsi, [pString2]
	:
	mov al, [rsi]
	xlatb
	mov ah,al
	mov al, [rdi]
	xlatb
	add rsi,1
	add rdi,1
	sub al, ah
	jnz >
	or ah, ah
	jnz <
	:
	movsx rax, al
	test rax,rax
	ret

ENDF

RDBG_ASRT_push FRAME p1
	uses rbx

	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	mov ecx,D[rbx+BUFFERPOINTERS.StackIndex]
	movsxd rcx,ecx
	cmp rcx,STACKSIZE
	jge >>.ERROR

		shl rcx,4
		add rcx,rbx

		mov rdx,[p1]
		mov rax,[rdx+STACKENTRY.var]
		mov [rcx+STACKENTRY.var],rax
		mov rax,[rdx+STACKENTRY.sym]
		mov [rcx+STACKENTRY.sym],rax

		inc D[rbx+BUFFERPOINTERS.StackIndex]

		mov rax,1
		ret
	.ERROR
	mov rax,0
	ret
endf

RDBG_ASRT_pop FRAME p1
	uses rbx

	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	mov ecx,D[rbx+BUFFERPOINTERS.StackIndex]
	movsxd rcx,ecx
	cmp rcx,0
	jle >>.ERROR

		dec D[rbx+BUFFERPOINTERS.StackIndex]
		dec rcx
		shl rcx,4
		add rcx,rbx
		mov rdx,[p1]

		mov rax,[rcx+STACKENTRY.var]
		mov [rdx+STACKENTRY.var],rax
		mov rax,[rcx+STACKENTRY.sym]
		mov [rdx+STACKENTRY.sym],rax

		mov rax,1
		ret
	.ERROR
	mov rax,0
	ret
endf

RDBG_ASRT_peek FRAME p1
	uses rbx

	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	mov ecx,D[rbx+BUFFERPOINTERS.StackIndex]
	movsxd rcx,ecx
	cmp rcx,0
	jle >>.ERROR

		dec rcx
		shl rcx,4
		add rcx,rbx
		mov rdx,[p1]

		mov rax,[rcx+STACKENTRY.var]
		mov [rdx+STACKENTRY.var],rax
		mov rax,[rcx+STACKENTRY.sym]
		mov [rdx+STACKENTRY.sym],rax

		mov rax,1
		ret
	.ERROR
	mov rax,0
	ret
endf

RDBG_ASRT_initstack FRAME
	uses rbx
	LOCAL bullshot:D

	push rdi
	mov rcx,STACKSIZE
	shl rcx,3

	DB 0x65,0x48,0x8B,0x3C,0x25,0x28,0x00,0x00,0x00 ;gs mov rdi,[0x28]

	mov rax,0
	rep stosb
	pop rdi

	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	mov D[rbx+BUFFERPOINTERS.StackIndex],0

	ret
ENDF

RDBG_ASRT_binary_op FRAME sym
	uses ebx
	LOCAL temp:STACKENTRY


	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	cmp D[rbx+BUFFERPOINTERS.StackIndex],1
	jge >
		mov rax,CALC_FAIL
		movsxd rax,eax
		ret
	:
	invoke RDBG_ASRT_peek,offset temp
	cmp Q[temp.sym],CALC_SYMBOL_NUMBER
	je >
		mov rax,CALC_FAIL
		movsxd rax,eax
		ret
	:
	mov rax,[sym]
	mov [temp.sym],rax
	invoke RDBG_ASRT_push,offset temp
	test rax,rax
	jnz >
		mov rax,CALC_FAIL
		movsxd rax,eax
		ret
	:
	mov rax,CALC_SUCCESS
	movsxd rax,eax
	ret
endf

RDBG_ASRT_unary_op FRAME sym
	LOCAL temp:STACKENTRY

	mov rax,[sym]
	mov [temp.sym],rax

	invoke RDBG_ASRT_push,offset temp
	test rax,rax
	jnz >
		mov rax,CALC_FAIL
		movsxd rax,eax
		ret
	:

	mov rax,CALC_SUCCESS
	movsxd rax,eax
	ret
endf

RDBG_ASRT_paren FRAME sym
	uses rbx
	LOCAL temp:STACKENTRY
	LOCAL result:Q

	mov rbx,[sym]
	cmp rbx,CALC_SYMBOL_OPEN
	jne >.CALC_SYMBOL_CLOSE
		mov [temp.sym],rbx
		invoke RDBG_ASRT_push,offset temp
		test rax,rax
		jz >>.ERROR
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret

	.CALC_SYMBOL_CLOSE
	cmp rbx,CALC_SYMBOL_CLOSE
	jne >>.ERROR
		invoke RDBG_ASRT_pop,offset temp
		test rax,rax
		jz >>.ERROR
		cmp Q[temp.sym],CALC_SYMBOL_NUMBER
		jne >>.ERROR
		mov rax,[temp.var]
		mov [result],rax
		invoke RDBG_ASRT_pop,offset temp
		test rax,rax
		jz >>.ERROR
		cmp Q[temp.sym],CALC_SYMBOL_OPEN
		jne >>.ERROR
		invoke RDBG_ASRT_number,[result]
		cmp rax,CALC_SUCCESS
		jne >>.ERROR
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret

	.ERROR
	mov rax,CALC_FAIL
	movsxd rax,eax
	ret
endf

RDBG_ASRT_number FRAME num
	uses rbx,rsi
	LOCAL temp:STACKENTRY
	LOCAL signed:Q
	LOCAL precede:Q // CALC_SYMBOLS
	LOCAL result:Q

	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	mov eax,D[rbx+BUFFERPOINTERS.SignedOperations]
	and rax,0xffffffff
	mov [signed],rax
	cmp D[rbx+BUFFERPOINTERS.StackIndex],1
	jge >>
		mov rax,[num]
		mov [temp.var],rax
		mov Q[temp.sym],CALC_SYMBOL_NUMBER
		invoke RDBG_ASRT_push,offset temp
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:

	invoke RDBG_ASRT_peek,offset temp
	mov rbx,[temp.sym]
	mov [precede],rbx

	cmp rbx,CALC_SYMBOL_NOT
	jge >>.CALC_SYMBOL_NOT
		// OR,AND,XOR,*,/,+,-
		invoke RDBG_ASRT_pop,offset temp
		test rax,rax
		jnz >
			mov rax,CALC_FAIL
			movsxd rax,eax
			ret
		:
		invoke RDBG_ASRT_pop,offset temp
		test rax,rax
		jnz >
			mov rax,CALC_FAIL
			movsxd rax,eax
			ret
		:
		cmp Q[temp.sym],CALC_SYMBOL_NUMBER
		je >>
			mov rax,CALC_FAIL
			movsxd rax,eax
			ret
		:
		// Now the calculation
		cmp rbx,CALC_SYMBOL_OR
		jne >.CALC_SYMBOL_AND
			mov rax,[temp.var]
			mov rcx,[num]
			or rax,rcx
			mov [temp.var],rax
			jmp >>.CALCEND

		.CALC_SYMBOL_AND
		cmp rbx,CALC_SYMBOL_AND
		jne >.CALC_SYMBOL_XOR
			mov rax,[temp.var]
			mov rcx,[num]
			and rax,rcx
			mov [temp.var],rax
			jmp >>.CALCEND
		
		.CALC_SYMBOL_XOR
		cmp rbx,CALC_SYMBOL_XOR
		jne >.CALC_SYMBOL_MULTIPLY
			mov rax,[temp.var]
			mov rcx,[num]
			xor rax,rcx
			mov [temp.var],rax
			jmp >>.CALCEND
		
		.CALC_SYMBOL_MULTIPLY
		cmp rbx,CALC_SYMBOL_MULTIPLY
		jne >.CALC_SYMBOL_DIVIDE
			mov rax,[temp.var]
			mov rcx,[num]
			xor rdx,rdx
			cmp Q[signed],TRUE
			jne >.UNSIGNEDMUL
				imul rax,rcx
				jmp >
			.UNSIGNEDMUL
				mul rcx
			:
			test rdx,rdx
			jz >
				mov rax,CALC_OVERFLOW
				movsxd rax,eax
				ret
			:
			mov [temp.var],rax
			jmp >>.CALCEND

		.CALC_SYMBOL_DIVIDE
		cmp rbx,CALC_SYMBOL_DIVIDE
		jne >>.CALC_SYMBOL_ADD
			cmp Q[num],0
			jne >
				mov rax,CALC_DIVBYZERO
				movsxd rax,eax
				ret
			:
			mov rax,[temp.var]
			mov rcx,[num]
			xor rdx,rdx
			cmp Q[signed],TRUE
			jne >.UNSIGNEDDIV
				idiv rcx
				jmp >
			.UNSIGNEDDIV
				div rcx
			:
			mov [temp.var],rax
			jmp >>.CALCEND

		.CALC_SYMBOL_ADD
		cmp rbx,CALC_SYMBOL_ADD
		jne >.CALC_SYMBOL_SUBTRACT
			mov rax,[temp.var]
			mov rcx,[num]
			add rax,rcx
			mov [temp.var],rax
			jmp >>.CALCEND

		.CALC_SYMBOL_SUBTRACT
		cmp rbx,CALC_SYMBOL_SUBTRACT
		jne >>.DEFAULT
			mov rax,[temp.var]
			mov rcx,[num]
			sub rax,rcx
			mov [temp.var],rax
			jmp >>.CALCEND

		.CALCEND
			// Push the result onto the stack
			mov Q[temp.sym],CALC_SYMBOL_NUMBER
			invoke RDBG_ASRT_push,offset temp
			test rax,rax
			jnz >
				mov rax,CALC_FAIL
				movsxd rax,eax
				ret
			:
			mov rax,CALC_SUCCESS
			movsxd rax,eax
			ret

	.CALC_SYMBOL_NOT
	cmp ebx,CALC_SYMBOL_NOT
	jne >>.CALC_SYMBOL_OPEN
		invoke RDBG_ASRT_pop,offset temp
		mov Q[result],TRUE
		cmp Q[num],0
		je >
			mov Q[result],FALSE
		:
		invoke RDBG_ASRT_number,[result]
		ret

	.CALC_SYMBOL_OPEN
	cmp ebx,CALC_SYMBOL_OPEN
	jne >>.CALC_SYMBOL_CLOSE
		mov rax,[num]
		mov [temp.var],rax
		mov Q[temp.sym],CALC_SYMBOL_NUMBER
		invoke RDBG_ASRT_push,offset temp
		test rax,rax
		jnz >
			mov rax,CALC_FAIL
			movsxd rax,eax
			ret
		:
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret

	.CALC_SYMBOL_CLOSE
	cmp rbx,CALC_SYMBOL_CLOSE
	jne >>.CALC_SYMBOL_NUMBER
		// Not possible
		mov rax,CALC_FAIL
		movsxd rax,eax
		ret

	.CALC_SYMBOL_NUMBER
	cmp rbx,CALC_SYMBOL_NUMBER
	jne >>.DEFAULT
		// Not possible
		mov rax,CALC_FAIL
		movsxd rax,eax
		ret

	.DEFAULT
		mov rax,CALC_FAIL
		movsxd rax,eax
		ret

	ret
endf

RDBG_ASRT_calculate FRAME exp,result
	uses edi,esi,ebx
	LOCAL temp:STACKENTRY
	LOCAL szNumber[64]:B
	LOCAL dqNumber:Q

	call RDBG_ASRT_initstack
	
	mov rdi,[exp]
	test rdi,rdi
	jnz >
		// bad pointer return 0
		mov rax,0
		ret
	:
	mov al,[rdi]
	test al,al
	jnz >
		// empty string
		mov eax,0
		ret
	:
	// set the index to zero
	xor rbx,rbx
	dec rbx
	.WALKEXPRESSION
		inc rbx
		mov rsi,[rdi+rbx]
		mov rcx,rsi
		and rsi,0xff
		test rsi,rsi
		jz >>.ENDWALK

		.OR
		cmp rsi,'|'
		jne >>.AND
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_OR
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			// If it fails return the index
			mov rax,rbx
			ret

		.AND
		cmp rsi,'&'
		jne >>.XOR
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_AND
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.XOR
		cmp rsi,'^'
		jne >>.NOT
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_XOR
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.NOT
		cmp rsi,'!'
		jne >>.OPEN
			invoke RDBG_ASRT_unary_op,CALC_SYMBOL_NOT
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.OPEN
		cmp rsi,'('
		jne >>.CLOSE
			invoke RDBG_ASRT_paren,CALC_SYMBOL_OPEN
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.CLOSE
		cmp rsi,')'
		jne >>.MULTIPLY
			invoke RDBG_ASRT_paren,CALC_SYMBOL_CLOSE
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.MULTIPLY
		cmp rsi,'*'
		jne >>.DIVIDE
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_MULTIPLY
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.DIVIDE
		cmp rsi,'/'
		jne >>.ADD
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_DIVIDE
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.ADD
		cmp rsi,'+'
		jne >>.SUBTRACT
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_ADD
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.SUBTRACT
		cmp esi,'-'
		jne >>.SPACE

			DB 0x65,0x48,0x8b,0x14,0x25,0x28,0x00,0x00,0x00 ;gs mov rdx,[0x28]
			cmp D[rdx+BUFFERPOINTERS.StackIndex],0
			je >>.NUMBER
			invoke RDBG_ASRT_peek,offset temp
			cmp Q[temp.sym],CALC_SYMBOL_NUMBER
			jne >>.NUMBER
			invoke RDBG_ASRT_binary_op,CALC_SYMBOL_SUBTRACT
			cmp eax,CALC_FAIL
			jne <<.WALKEXPRESSION
			mov rax,rbx
			ret

		.SPACE
		cmp rsi," "
		je <<.WALKEXPRESSION

		.TAB
		cmp rsi,9
		je <<.WALKEXPRESSION

		.NUMBER
			mov rax,offset szNumber
			cmp rsi,"0"
			jne >
				// This may be a HEX number check the next character for X or x
				// ECX contains the next 4 bytes of the expression
				shr rcx,8
				and rcx,0xFF
				or rcx,0x20
				cmp rcx,"x"
				je >>.ISHEXNUMBER
			:
			cmp rsi,"-"
			jne >
				mov B[rax],"-"
				inc rax
				inc rbx
				mov rsi,[rdi+rbx]
				and rsi,0xff
			:
			cmp rsi,0x30
			jl >>.SYMBOL
			cmp rsi,0x39
			jg >>.SYMBOL
			// we have a number copy it to the buffer
				:
				mov cl,[rdi+rbx]
				cmp cl,0x39
				jg >
				cmp cl,0x30
				jl >
				mov [rax],cl
				inc rbx
				inc rax
				jmp <
				:
			dec rbx
			mov B[rax],0
			invoke StrToInt64ExA,offset szNumber,0,offset dqNumber
			invoke RDBG_ASRT_number,[dqNumber]
			cmp rax,CALC_SUCCESS

			je <<.WALKEXPRESSION
			mov rcx,[result]
			mov Q[rcx],rax
			mov rax,rbx
			ret

			.ISHEXNUMBER
			// The number is in HEX format copy it to the buffer
			// call the hex convertor. On return the number will be in EAX
			// the length will be in ECX, ECX will be -1 if an invalid number
			// has been entered (ie > 0xFFFFFFFF or < 0x0)

			// Skip past the 0x
			add rbx,2
			mov rcx,rdi
			add rcx,rbx

			invoke RDBG_ASRT_ConvertHex,rcx
			test rcx,rcx
			js >>.INVALID
			add rbx,rcx
			dec rbx

			invoke RDBG_ASRT_number,rax
			cmp rax,CALC_SUCCESS

			je <<.WALKEXPRESSION
			mov rcx,[result]
			mov Q[rcx],rax
			mov rax,rbx
			ret

		.SYMBOL
			// the first letter didn't match any of the normal expected ones
			// so we'll see if it's in the symbol table

			// Symbols must start with a letter
			// To get the full symbol name scan ahead until
			// we find an operator. This will be done in a subroutine
			// it will return CALC_SUCCESS if the symbol is found
			// CALC_FAIL if not found
			// The current value of the symbol is in temp.var
			// ecx will contain the count of characters in the symbol so
			// the index can be advanced
			lea rax,[rdi+rbx]
			invoke RDBG_ASRT_ScanForSymbol, rax, offset temp
			cmp eax,CALC_FAIL
			jne >
				mov rax,CALC_SYMNOTFOUND
				ret
			:
			add rbx,rcx
			dec rbx
			invoke RDBG_ASRT_number,[temp.var]
			cmp rax,CALC_SUCCESS
			je <<.WALKEXPRESSION

			mov rcx,[result]
			mov Q[rcx],rax
			mov rax,rbx
			ret

		.INVALID
			mov rax,rbx
			ret

	.ENDWALK

	DB 0x65,0x48,0x8b,0x1c,0x25,0x28,0x00,0x00,0x00 ;gs mov rbx,[0x28]
	cmp D[rbx+BUFFERPOINTERS.StackIndex],1
	je >
		mov rax,rbx
		ret
	:
	invoke RDBG_ASRT_pop,offset temp
	test rax,rax
	jnz >
		mov rax,rbx
		ret
	:
	cmp D[temp.sym],CALC_SYMBOL_NUMBER
	je >
		mov rax,rbx
		ret
	:
	mov rax,[result]
	mov rcx,[temp.var]
	mov [rax],rcx
	mov rax,CALC_SUCCESS
	movsxd rax,eax
	ret
endf

RDBG_ASRT_ConvertHex FRAME pStartOfHex
	uses rdi,rsi,rbx
	LOCAL pszHex[256]:B
	LOCAL intRet:Q

	xor rbx,rbx
	mov rsi,[pStartOfHex]
	lea rdi,pszHex

	// Copy the 0x
	mov rax,"0x"
	mov [rdi],rax
	add rdi,2
	xor rax,rax

	:
	mov al,[rsi+rbx]
	cmp al,"0"
	jl >.BADCHAR
	cmp al,"9"
	jle >.GOODCHAR
	or al,0x20

	cmp al,"a"
	jl >.BADCHAR
	cmp al,"f"
	jg >.BADCHAR

	.GOODCHAR
	mov [rdi+rbx],al
	inc rbx
	jmp <

	.BADCHAR
	mov B[rdi+rbx],0
	// The string is packed
	// check the length
	cmp rbx,16
	jg >>.INVALIDNUMBER
	test rbx,rbx
	jz >>.INVALIDNUMBER

	// Push the length
	push rbx

	invoke StrToInt64ExA,offset pszHex,1,offset intRet

	// Pop the length
	pop rcx
	test rax,rax
	jz >.INVALIDNUMBER

	mov rax,[intRet]
	ret

	.INVALIDNUMBER
	xor rax,rax
	mov rcx,-1
	ret

endf

RDBG_ASRT_ScanForSymbol FRAME pExpr, p1
	uses rbx,rdi,rsi
	LOCAL szSymbolName[256]:B
	LOCAL pSymbol:Q
	LOCAL fByValue:Q
	LOCAL DataType:Q
	LOCAL RcxTemp:Q

	mov Q[DataType],DATATYPE_DEFAULT

	// Optional type indicators (B,W,D) (BYTE,WORD and DWORD respectively)
	// types can only be used on bracketed labels []. Any other use will
	// cause the evaluation to fail. The open bracket must be immediately
	// following the type indicator, no spaces are allowed between them

	// Brackets are not allowed with registers (yet)

	// no brackets [] indicate that the address is to
	// be used instead of the value when dealing with symbols
	// registers will always return the value and cannot be
	// dereferenced (yet)

	mov Q[fByValue],FALSE

	// Scan ahead to the next whitespace or operator
	mov rdi,[pExpr]
	
	mov al,[rdi]
	cmp al,"["
	jne >
		mov Q[fByValue],TRUE
		inc rdi
		jmp >>.BREAK
	:
	// B W D
	cmp al,"B"
	jne >
		mov al,[rdi+1]
		cmp al,"["
		jne >
			mov Q[fByValue],TRUE
			mov Q[DataType],DATATYPE_BYTE
			add rdi,2
			jmp >>.BREAK
	:
	cmp al,"W"
	jne >
		mov al,[rdi+1]
		cmp al,"["
		jne >
			mov Q[fByValue],TRUE
			mov Q[DataType],DATATYPE_WORD
			add rdi,2
			jmp >>.BREAK
	:
	cmp al,"D"
	jne >
		mov al,[rdi+1]
		cmp al,"["
		jne >.BREAK
			mov Q[fByValue],TRUE
			mov Q[DataType],DATATYPE_DWORD
			add rdi,2
	:
	cmp al,"Q"
	jne >.BREAK
		mov al,[rdi+1]
		cmp al,"["
		jne >.BREAK
			mov Q[fByValue],TRUE
			mov Q[DataType],DATATYPE_QWORD
			add rdi,2
	.BREAK

	push rdi
	xor rcx,rcx

	:
		mov al,[rdi]
		inc rdi
		inc rcx
		test al,al
		jz >
		cmp al,"+"
		je >
		cmp al,"-"
		je >
		cmp al,"*"
		je >
		cmp al,"/"
		je >
		cmp al,"&"
		je >
		cmp al,"|"
		je >
		cmp al,"^"
		je >
		cmp al,"!"
		je >
		cmp al," "
		je >
		cmp al,9
		je >
		cmp al,"]"
		jne <
	:

	pop rsi // ESI points to the source string for movsb
	dec rcx // the last character is the delimiter, don't copy it
	push rcx // Save the length of the symbol for later
	lea rdi,szSymbolName // EDI points to the destination string for movsb
	rep movsb // copy the symbol name to a temporary buffer
	mov B[rdi],0 // NULL terminate the string

	// Check to see if the symbol is TRUE or FALSE
	// They are supported as FALSE = 0, TRUE = 1
	invoke lstrcmpi,offset szSymbolName,"TRUE"
	test rax,rax
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov Q[rdx+STACKENTRY.var],1
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"FALSE"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov Q[rdx+STACKENTRY.var],0
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:

	// Scan for register names
	// Registers are saved in a context structure before calculate is called

	// Get the offset of the register values
	DB 0x65,0x48,0x8B,0x3C,0x25,0x28,0x00,0x00,0x00 ;gs mov rdi,[0x28]
	add rdi,BUFFERPOINTERS.reg

	// 8 bit registers
	invoke lstrcmpi,offset szSymbolName,"AL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rax]
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"BL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbx]
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"CL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rcx]
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"DL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdx]
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"AH"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rax]
		shr rax,8
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"BH"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[edi+REGISTERS.Rbx]
		shr rax,8
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"CH"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[edi+REGISTERS.Rcx]
		shr rax,8
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"DH"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdx]
		shr rax,8
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"SPL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsp]
		shr rax,8
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"BPL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbp]
		shr rax,8
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"SIL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsi]
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"DIL"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdi]
		and rax,0xFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:

	// 16 bit registers
	invoke lstrcmpi,offset szSymbolName,"AX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rax]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"BX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbx]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"CX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rcx]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"DX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdx]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"SI"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsi]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"DI"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdi]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"SP"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsp]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"BP"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbp]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R8W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R8]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R9W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R9]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R10W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R10]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R11W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R11]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R12W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R12]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R13W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R13]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R14W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R14]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R15W"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R15]
		and rax,0xFFFF
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:

	// 32 bit registers
	invoke lstrcmpi,offset szSymbolName,"EAX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rax]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"EBX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbx]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"ECX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rcx]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"EDX"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdx]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"ESI"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsi]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"EDI"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdi]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"ESP"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsp]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"EBP"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbp]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R8D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R8]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R9D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R9]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R10D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R10]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R11D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R11]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R12D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R12]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R13D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R13]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R14D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R14]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R15D"
	test rax,rax
	jnz >
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R15]
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	
	// 64 bit registers
	invoke lstrcmpi,offset szSymbolName,"RAX"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rax]
		cmp Q[fByValue],FALSE
		je >.RAXBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RAXWORD
			and rax,0xFF
			inc rcx
			jmp >.RAXBYVALUE
		.RAXWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RAXDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RAXBYVALUE
		.RAXDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RAXQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RAXBYVALUE
		.RAXQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RAXBYVALUE
			inc rcx
		.RAXBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"RBX"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbx]
		cmp Q[fByValue],FALSE
		je >.RBXBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RBXWORD
			and rax,0xFF
			inc rcx
			jmp >.RBXBYVALUE
		.RBXWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RBXDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RBXBYVALUE
		.RBXDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RBXQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RBXBYVALUE
		.RBXQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RBXBYVALUE
			inc rcx
		.RBXBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"RCX"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rcx]
		cmp Q[fByValue],FALSE
		je >.RCXBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RCXWORD
			and rax,0xFF
			inc rcx
			jmp >.RCXBYVALUE
		.RCXWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RCXDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RCXBYVALUE
		.RCXDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RCXQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RCXBYVALUE
		.RCXQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RCXBYVALUE
			inc rcx
		.RCXBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"RDX"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdx]
		cmp Q[fByValue],FALSE
		je >.RDXBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RDXWORD
			and rax,0xFF
			inc rcx
			jmp >.RDXBYVALUE
		.RDXWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RDXDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RDXBYVALUE
		.RDXDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RDXQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RDXBYVALUE
		.RDXQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RDXBYVALUE
			inc rcx
		.RDXBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"RDI"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rdi]
		cmp Q[fByValue],FALSE
		je >.RDIBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RDIWORD
			and rax,0xFF
			inc rcx
			jmp >.RDIBYVALUE
		.RDIWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RDIDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RDIBYVALUE
		.RDIDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RDIQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RDIBYVALUE
		.RDIQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RDIBYVALUE
			inc rcx
		.RDIBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"RSI"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsi]
		cmp Q[fByValue],FALSE
		je >.RSIBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RSIWORD
			and rax,0xFF
			inc rcx
			jmp >.RSIBYVALUE
		.RSIWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RSIDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RSIBYVALUE
		.RSIDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RSIQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RSIBYVALUE
		.RSIQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RSIBYVALUE
			inc rcx
		.RSIBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"RBP"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rbp]
		cmp Q[fByValue],FALSE
		je >.RBPBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RBPWORD
			and rax,0xFF
			inc rcx
			jmp >.RBPBYVALUE
		.RBPWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RBPDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RBPBYVALUE
		.RBPDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RBPQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RBPBYVALUE
		.RBPQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RBPBYVALUE
			inc rcx
		.RBPBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"RSP"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.Rsp]
		cmp Q[fByValue],FALSE
		je >.RSPBYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.RSPWORD
			and rax,0xFF
			inc rcx
			jmp >.RSPBYVALUE
		.RSPWORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.RSPDWORD
			and rax,0xFFFF
			inc rcx
			jmp >.RSPBYVALUE
		.RSPDWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.RSPQWORD
			and rax,0xffffffff
			inc rcx
			jmp >.RSPBYVALUE
		.RSPQWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.RSPBYVALUE
			inc rcx
		.RSPBYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R8"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R8]
		cmp Q[fByValue],FALSE
		je >>.R8BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R8WORD
			and rax,0xFF
			inc rcx
			jmp >.R8BYVALUE
		.R8WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R8DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R8BYVALUE
		.R8DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R8QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R8BYVALUE
		.R8QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R8BYVALUE
			inc rcx
		.R8BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R9"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R9]
		cmp Q[fByValue],FALSE
		je >.R9BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R9WORD
			and rax,0xFF
			inc rcx
			jmp >.R9BYVALUE
		.R9WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R9DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R9BYVALUE
		.R9DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R9QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R9BYVALUE
		.R9QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R9BYVALUE
			inc rcx
		.R9BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R10"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R10]
		cmp Q[fByValue],FALSE
		je >.R10BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R10WORD
			and rax,0xFF
			inc rcx
			jmp >.R10BYVALUE
		.R10WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R10DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R10BYVALUE
		.R10DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R10QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R10BYVALUE
		.R10QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R10BYVALUE
			inc rcx
		.R10BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R11"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R11]
		cmp Q[fByValue],FALSE
		je >.R11BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R11WORD
			and rax,0xFF
			inc rcx
			jmp >.R11BYVALUE
		.R11WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R11DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R11BYVALUE
		.R11DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R11QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R11BYVALUE
		.R11QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R11BYVALUE
			inc rcx
		.R11BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R12"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R12]
		cmp Q[fByValue],FALSE
		je >.R12BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R12WORD
			and rax,0xFF
			inc rcx
			jmp >.R12BYVALUE
		.R12WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R12DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R12BYVALUE
		.R12DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R12QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R12BYVALUE
		.R12QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R12BYVALUE
			inc rcx
		.R12BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R13"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R13]
		cmp Q[fByValue],FALSE
		je >.R13BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R13WORD
			and rax,0xFF
			inc rcx
			jmp >.R13BYVALUE
		.R13WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R13DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R13BYVALUE
		.R13DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R13QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R13BYVALUE
		.R13QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R13BYVALUE
			inc rcx
		.R13BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R14"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R14]
		cmp Q[fByValue],FALSE
		je >.R14BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R14WORD
			and rax,0xFF
			inc rcx
			jmp >.R14BYVALUE
		.R14WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R14DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R14BYVALUE
		.R14DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R14QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R14BYVALUE
		.R14QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R14BYVALUE
			inc rcx
		.R14BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	invoke lstrcmpi,offset szSymbolName,"R15"
	test rax,rax
	jnz >>
		pop rcx
		mov rdx,[p1]
		mov rax,[rdi+REGISTERS.R15]
		cmp Q[fByValue],FALSE
		je >.R15BYVALUE
			mov rax,[rax]
			add rcx,2
		cmp Q[DataType],DATATYPE_BYTE
		jne >>.R15WORD
			and rax,0xFF
			inc rcx
			jmp >.R15BYVALUE
		.R15WORD
		cmp Q[DataType],DATATYPE_WORD
		jne >>.R15DWORD
			and rax,0xFFFF
			inc rcx
			jmp >.R15BYVALUE
		.R15DWORD
		cmp Q[DataType],DATATYPE_DWORD
		jne >>.R15QWORD
			and rax,0xffffffff
			inc rcx
			jmp >.R15BYVALUE
		.R15QWORD
		cmp Q[DataType],DATATYPE_QWORD
		jne >>.R15BYVALUE
			inc rcx
		.R15BYVALUE
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:

	.SKIPSEARCH

	// We need the length of the symbol again so pop it into ECX
	pop [RcxTemp]

	invoke RDBG_ASRT_FindSymbolByName,-1,offset szSymbolName,offset pSymbol,[RcxTemp]
	mov rcx,[RcxTemp]
	test rax,rax // Returns 0 for success, -1 if the sym isn't found
	js >>.ERROR
	mov rax,[pSymbol]
	cmp D[fByValue],FALSE
	je >
		// The value not the address is required, dereference it and
		// adjust the value of the character count to reflect it
		mov rax,[rax]
		add rcx,2 // skip over the two [] brackets
	:

	cmp Q[DataType],DATATYPE_BYTE
	jne >
		mov rdx,[p1]
		and rax,0xFF
		inc rcx
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	cmp D[DataType],DATATYPE_WORD
	jne >
		mov rdx,[p1]
		and rax,0xFFFF
		inc rcx
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	cmp D[DataType],DATATYPE_DWORD
	jne >
		mov rdx,[p1]
		inc rcx
		and rax,0xffffffff
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:
	cmp D[DataType],DATATYPE_QWORD
	jne >
		mov rdx,[p1]
		inc rcx
		mov [rdx+STACKENTRY.var],rax
		mov rax,CALC_SUCCESS
		movsxd rax,eax
		ret
	:

	// DATATYPE_DEFAULT
	// Put the value into the structure and return success
	// On exit RCX will contain the character count
	mov rdx,[p1]
	mov [rdx+STACKENTRY.var],rax
	mov rax,CALC_SUCCESS
	movsxd rax,eax
	ret

	.ERROR
	mov rax,CALC_FAIL
	movsxd rax,eax
	ret
ENDF

RDBG_ASRT_FindSymbolByName FRAME hProcess,pszSymbol,ppAddress,lenSymbol
	uses rbx,rdi,rsi
	LOCAL pSymInfo:Q
	LOCAL SymModInfo:IMAGEHLP_MODULE64
	LOCAL ProcessPath[256]:B
	LOCAL hFile:Q
	LOCAL ihmod64:IMAGEHLP_MODULE64
	LOCAL ModuleBase:Q
	LOCAL modinfo:MODULEINFO

	invoke CoTaskMemAlloc,SIZEOF SYMBOL_INFO+MAX_SYM_NAME
	mov [pSymInfo],rax

	mov rbx,[lenSymbol]
	mov [rax+SYMBOL_INFO.NameLen],ebx
	mov D[rax+SYMBOL_INFO.MaxNameLen],MAX_SYM_NAME
	mov D[rax+SYMBOL_INFO.SizeOfStruct],SIZEOF SYMBOL_INFO

	mov rbx,[pSymInfo]
	add rbx,SYMBOL_INFO.Name
	invoke lstrcpyA,rbx,[pszSymbol]

	invoke GetModuleFileName,NULL,offset ProcessPath,MAX_PATH

	invoke GetModuleHandle,NULL

	invoke GetModuleInformation,[hProcess],rax,offset modinfo,SIZEOF MODULEINFO

	invoke CreateFile,offset ProcessPath,GENERIC_READ,NULL,NULL,OPEN_EXISTING,NULL,NULL
	mov [hFile],rax

	invoke SymInitialize,[hProcess], NULL, FALSE

	invoke SymLoadModule64,[hProcess],[hFile],offset ProcessPath,NULL,[modinfo.lpBaseOfDll],0
	mov [ModuleBase],rax

	mov D[ihmod64.SizeOfStruct],SIZEOF IMAGEHLP_MODULE64
	invoke SymGetModuleInfo64,[hProcess],[ModuleBase],offset ihmod64

	cmp D[ihmod64.SymType],SymNone
	je >>.NOSYM

	invoke SymFromName,[hProcess],rbx,[pSymInfo]
	test rax,rax
	jz >>.NOSYM
	mov rax,[pSymInfo]
	mov rax,[rax+SYMBOL_INFO.Address]
	mov rcx,[ppAddress]
	mov [rcx],rax

	// Shut it down
	invoke SymCleanup,[hProcess]
	invoke CoTaskMemFree,[pSymInfo]
	invoke CloseHandle,[hFile]
	xor rax,rax
	RET

	.NOSYM
	invoke SymCleanup,[hProcess]
	invoke CoTaskMemFree,[pSymInfo]
	invoke CloseHandle,[hFile]
	mov rcx,[ppAddress]
	mov Q[rcx],0
	xor rax,rax
	dec rax
	RET
endf
